@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkConfig.c 1.52 96/02/15 18:52:39
+ * SCCS: @(#) tkConfig.c 1.53 96/04/26 10:29:31
  */
 
 #include "tkPort.h"
@@ -127,6 +127,7 @@ Tk_ConfigureWidget(interp, tkwin, specs, argc, argv, widgRec, flags)
 	if (specPtr == NULL) {
 	    return TCL_ERROR;
 	}
+
 	/*
 	 * Process the entry.
 	 */
@@ -136,11 +137,11 @@ Tk_ConfigureWidget(interp, tkwin, specs, argc, argv, widgRec, flags)
 		    "\" missing", (char *) NULL);
 	    return TCL_ERROR;
 	}
-		
-	if ((flags & TK_CONFIG_CHECK_MY_FLAG) &&
-	    (specPtr->specFlags & TK_CONFIG_DONT_CHANGE_DEFAULT)) {
-	    continue;
-	}
+
+        if ((flags & TK_CONFIG_CHECK_MY_FLAG) &&
+            (specPtr->specFlags & TK_CONFIG_DONT_CHANGE_DEFAULT)) {
+            continue;
+        }
 
 	if (DoConfig(interp, tkwin, specPtr, argv[1], 0, widgRec) != TCL_OK) {
 	    char msg[100];
@@ -173,7 +174,7 @@ Tk_ConfigureWidget(interp, tkwin, specs, argc, argv, widgRec, flags)
 	    }
 	    value = NULL;
 	    if ((specPtr->dbName != NULL) && !((flags & TK_CONFIG_CHECK_MY_FLAG)
-		&& (specPtr->specFlags & TK_CONFIG_DONT_CHANGE_DEFAULT))) {
+                && (specPtr->specFlags & TK_CONFIG_DONT_CHANGE_DEFAULT))) {
 		value = Tk_GetOption(tkwin, specPtr->dbName, specPtr->dbClass);
 	    }
 	    if (value != NULL) {
@@ -408,22 +409,18 @@ DoConfig(interp, tkwin, specPtr, value, valueIsUid, widgRec)
 		break;
 	    }
 	    case TK_CONFIG_FONT: {
-		XFontStruct *newPtr, *oldPtr;
+		Tk_Font new;
 
 		if (nullValue) {
-		    newPtr = NULL;
+		    new = NULL;
 		} else {
-		    uid = valueIsUid ? (Tk_Uid) value : Tk_GetUid(value);
-		    newPtr = Tk_GetFontStruct(interp, tkwin, uid);
-		    if (newPtr == NULL) {
+		    new = Tk_GetFont(interp, tkwin, value);
+		    if (new == NULL) {
 			return TCL_ERROR;
 		    }
 		}
-		oldPtr = *((XFontStruct **) ptr);
-		if (oldPtr != NULL) {
-		    Tk_FreeFontStruct(oldPtr);
-		}
-		*((XFontStruct **) ptr) = newPtr;
+		Tk_FreeFont(*((Tk_Font *) ptr));
+		*((Tk_Font *) ptr) = new;
 		break;
 	    }
 	    case TK_CONFIG_BITMAP: {
@@ -800,9 +797,9 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, buffer, freeProcPtr)
 	    break;
 	}
 	case TK_CONFIG_FONT: {
-	    XFontStruct *fontStructPtr = *((XFontStruct **) ptr);
-	    if (fontStructPtr != NULL) {
-		result = Tk_NameOfFontStruct(fontStructPtr);
+	    Tk_Font tkfont = *((Tk_Font *) ptr);
+	    if (tkfont != NULL) {
+		result = Tk_NameOfFont(tkfont);
 	    }
 	    break;
 	}
@@ -974,10 +971,8 @@ Tk_FreeOptions(specs, widgRec, display, needFlags)
 		}
 		break;
 	    case TK_CONFIG_FONT:
-		if (*((XFontStruct **) ptr) != NULL) {
-		    Tk_FreeFontStruct(*((XFontStruct **) ptr));
-		    *((XFontStruct **) ptr) = NULL;
-		}
+		Tk_FreeFont(*((Tk_Font *) ptr));
+		*((Tk_Font *) ptr) = NULL;
 		break;
 	    case TK_CONFIG_BITMAP:
 		if (*((Pixmap *) ptr) != None) {
