@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkEvent.c 1.18 96/09/12 09:25:22
+ * SCCS: @(#) tkEvent.c 1.20 96/09/20 09:33:38
  */
 
 #include "tkPort.h"
@@ -484,6 +484,7 @@ Tk_HandleEvent(eventPtr)
      * are the same.  Compare the two fields and convert StructureNotify
      * to SubstructureNotify if necessary.
      */
+
     handlerWindow = eventPtr->xany.window;
     mask = eventMasks[eventPtr->xany.type];
     if (mask == StructureNotifyMask) {
@@ -548,34 +549,16 @@ Tk_HandleEvent(eventPtr)
 	
 	/*
 	 * Redirect KeyPress and KeyRelease events to the focus window,
-	 * or ignore them entirely if there is no focus window.  Map the
-	 * x and y coordinates to make sense in the context of the focus
-	 * window, if possible (make both -1 if the map-from and map-to
-	 * windows don't share the same screen).
+	 * or ignore them entirely if there is no focus window.
 	 */
     
 	if (mask & (KeyPressMask|KeyReleaseMask)) {
-	    TkWindow *focusPtr;
-	    int winX, winY, focusX, focusY;
-    
 	    winPtr->dispPtr->lastEventTime = eventPtr->xkey.time;
-	    focusPtr = TkGetFocus(winPtr);
-	    if (focusPtr == NULL) {
+	    winPtr = TkFocusKeyEvent(winPtr, eventPtr);
+	    if (winPtr == NULL) {
                 Tcl_Release((ClientData) interp);
 		return;
 	    }
-	    if ((focusPtr->display != winPtr->display)
-		    || (focusPtr->screenNum != winPtr->screenNum)) {
-		eventPtr->xkey.x = -1;
-		eventPtr->xkey.y = -1;
-	    } else {
-		Tk_GetRootCoords((Tk_Window) winPtr, &winX, &winY);
-		Tk_GetRootCoords((Tk_Window) focusPtr, &focusX, &focusY);
-		eventPtr->xkey.x -= focusX - winX;
-		eventPtr->xkey.y -= focusY - winY;
-	    }
-	    eventPtr->xkey.window = focusPtr->window;
-	    winPtr = focusPtr;
 	}
     
 	/*
@@ -940,20 +923,19 @@ Tk_QueueWindowEvent(eventPtr, position)
  */
 
 void
-TkQueueEventForAllChildren(tkwin, eventPtr)
-    Tk_Window tkwin;	    /* Window to which event is sent. */
+TkQueueEventForAllChildren(winPtr, eventPtr)
+    TkWindow *winPtr;	    /* Window to which event is sent. */
     XEvent *eventPtr;	    /* The event to be sent. */
 {
-    TkWindow *winPtr, *childPtr;
+    TkWindow *childPtr;
 
-    winPtr = (TkWindow *) tkwin;
     eventPtr->xany.window = winPtr->window;
     Tk_QueueWindowEvent(eventPtr, TCL_QUEUE_TAIL);
     
     childPtr = winPtr->childList;
     while (childPtr != NULL) {
 	if (!Tk_IsTopLevel(childPtr)) {
-	    TkQueueEventForAllChildren((Tk_Window) childPtr, eventPtr);
+	    TkQueueEventForAllChildren(childPtr, eventPtr);
 	}
 	childPtr = childPtr->nextPtr;
     }
