@@ -12,11 +12,13 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkFrame.c 1.68 96/02/15 18:53:30
+ * SCCS: @(#) tkFrame.c 1.82 97/08/08 17:26:26
  */
 /*
- * TkSTEP modifications Copyright by Alfredo K. Kojima
- */ 
+ * TkSTEP modifications Copyright (c) Alfredo K. Kojima
+ * - surgery performed and implanted into Tk8.0 by Steve Murray
+ */
+
 
 #include "default.h"
 #include "tkPort.h"
@@ -52,6 +54,8 @@ typedef struct {
     char *colormapName;		/* Textual description of colormap for window,
 				 * from -colormap option.  Malloc-ed, may be
 				 * NULL. */
+    char *menuName;		/* Textual description of menu to use for
+				 * menubar. Malloc-ed, may be NULL. */
     Colormap colormap;		/* If not None, identifies a colormap
 				 * allocated for this window, which must be
 				 * freed when the window is deleted. */
@@ -75,23 +79,26 @@ typedef struct {
     char *takeFocus;		/* Value of -takefocus option;  not used in
 				 * the C code, but used by keyboard traversal
 				 * scripts.  Malloc'ed, but may be NULL. */
+    int isContainer;		/* 1 means this window is a container, 0 means
+				 * that it isn't. */
+    char *useThis;		/* If the window is embedded, this points to
+				 * the name of the window in which it is
+				 * embedded (malloc'ed).  For non-embedded
+				 * windows this is NULL. */
     int flags;			/* Various flags;  see below for
 				 * definitions. */
-#ifdef FULL_STEP
-    XFontStruct *fontPtr;	/* Information about text font, or NULL. */
-    char *text;			/* optional text to display on frame */
-    Tk_Justify justify;		/* position of the text (left, center, right)*/
+#ifdef ENABLE_STEP
+    Tk_Font tkfont;	        /* Information about text font, or NULL. */
+    char *text;			/* Optional text to display on frame. */
+    Tk_Justify justify;		/* Position of the text (left, center, right).*/
     XColor *normalFg;		/* Foreground color in normal mode. */
-
-    GC textGC;			/* GC for drawing text */
-    GC backGC;			/* GC for background */
-    int numChars;		/* # of characters in text. */
-    int textWidth;		/* Width needed to display text as requested,
-				 * in pixels. */
-    int textHeight;		/* Height needed to display text as requested,
-				 * in pixels. */
+    GC textGC;			/* GC for drawing text. */
+    GC backGC;			/* GC for background. */
+    int numChars;		/* Number of characters in text. */
+    Tk_FontMetrics fm;          /* Collection of font dimensions. */
+    int textWidth;              /* String width in pixels. */
     int textY;			/* vertical position of text relative to 
-				 * window's top */
+				 * window's top. */
 #endif
 } Frame;
 
@@ -137,6 +144,8 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-colormap", "colormap", "Colormap",
 	DEF_FRAME_COLORMAP, Tk_Offset(Frame, colormapName),
 	BOTH|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_BOOLEAN, "-container", "container", "Container",
+	DEF_FRAME_CONTAINER, Tk_Offset(Frame, isContainer), BOTH},
     {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
 	DEF_FRAME_CURSOR, Tk_Offset(Frame, cursor), BOTH|TK_CONFIG_NULL_OK},
     {TK_CONFIG_PIXELS, "-height", "height", "Height",
@@ -149,16 +158,19 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_PIXELS, "-highlightthickness", "highlightThickness",
 	"HighlightThickness",
 	DEF_FRAME_HIGHLIGHT_WIDTH, Tk_Offset(Frame, highlightWidth), BOTH},
+    {TK_CONFIG_STRING, "-menu", "menu", "Menu",
+	DEF_TOPLEVEL_MENU, Tk_Offset(Frame, menuName),
+	TOPLEVEL|TK_CONFIG_NULL_OK},
     {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
 	DEF_FRAME_RELIEF, Tk_Offset(Frame, relief), BOTH},
     {TK_CONFIG_STRING, "-screen", "screen", "Screen",
 	DEF_TOPLEVEL_SCREEN, Tk_Offset(Frame, screenName),
 	TOPLEVEL|TK_CONFIG_NULL_OK},
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
     {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *) NULL,
 	(char *) NULL, 0, FRAME},
     {TK_CONFIG_FONT, "-font", "font", "Font",
-	DEF_FRAME_FONT, Tk_Offset(Frame, fontPtr),FRAME},
+	DEF_FRAME_FONT, Tk_Offset(Frame, tkfont),FRAME},
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
 	DEF_FRAME_FG, Tk_Offset(Frame, normalFg), FRAME},
     {TK_CONFIG_STRING, "-text", "text", "Text",
@@ -169,6 +181,8 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_FRAME_TAKE_FOCUS, Tk_Offset(Frame, takeFocus),
 	BOTH|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_STRING, "-use", "use", "Use",
+	DEF_FRAME_USE, Tk_Offset(Frame, useThis), TOPLEVEL|TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-visual", "visual", "Visual",
 	DEF_FRAME_VISUAL, Tk_Offset(Frame, visualName),
 	BOTH|TK_CONFIG_NULL_OK},
@@ -271,9 +285,10 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
 {
     Tk_Window tkwin = (Tk_Window) clientData;
     Frame *framePtr;
-    Tk_Window new = NULL;
-    char *className, *screenName, *visualName, *colormapName, *arg;
+    Tk_Window new;
+    char *className, *screenName, *visualName, *colormapName, *arg, *useOption;
     int i, c, length, depth;
+    unsigned int mask;
     Colormap colormap;
     Visual *visual;
 
@@ -290,7 +305,7 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
      * is configured using the usual Tk mechanisms.
      */
 
-    className = colormapName = screenName = visualName = NULL;
+    className = colormapName = screenName = visualName = useOption = NULL;
     colormap = None;
     for (i = 2; i < argc; i += 2) {
 	arg = argv[i];
@@ -308,6 +323,9 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
 	} else if ((c == 's') && toplevel
 		&& (strncmp(arg, "-screen", strlen(arg)) == 0)) {
 	    screenName = argv[i+1];
+	} else if ((c == 'u') && toplevel
+		&& (strncmp(arg, "-use", strlen(arg)) == 0)) {
+	    useOption = argv[i+1];
 	} else if ((c == 'v')
 		&& (strncmp(arg, "-visual", strlen(arg)) == 0)) {
 	    visualName = argv[i+1];
@@ -315,10 +333,18 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
     }
 
     /*
-     * Create the window, and deal with the special options -classname,
-     * -colormap, -screenname, and -visual.  The order here is tricky,
-     * because we want to allow values for these options to come from
-     * the database, yet we can't do that until the window is created.
+     * Create the window, and deal with the special options -use,
+     * -classname, -colormap, -screenname, and -visual.  These options
+     * must be handled before calling ConfigureFrame below, and they must
+     * also be processed in a particular order, for the following
+     * reasons:
+     * 1. Must set the window's class before calling ConfigureFrame,
+     *    so that unspecified options are looked up in the option
+     *    database using the correct class.
+     * 2. Must set visual information before calling ConfigureFrame
+     *    so that colors are allocated in a proper colormap.
+     * 3. Must call TkpUseWindow before setting non-default visual
+     *    information, since TkpUseWindow changes the defaults.
      */
 
     if (screenName == NULL) {
@@ -346,6 +372,14 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
 	}
     }
     Tk_SetClass(new, className);
+    if (useOption == NULL) {
+	useOption = Tk_GetOption(new, "use", "Use");
+    }
+    if (useOption != NULL) {
+	if (TkpUseWindow(interp, new, useOption) != TCL_OK) {
+	    goto error;
+	}
+    }
     if (visualName == NULL) {
 	visualName = Tk_GetOption(new, "visual", "Visual");
     }
@@ -407,21 +441,42 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
     framePtr->height = 0;
     framePtr->cursor = None;
     framePtr->takeFocus = NULL;
+    framePtr->isContainer = 0;
+    framePtr->useThis = NULL;
     framePtr->flags = 0;
-#ifdef FULL_STEP
+    framePtr->menuName = NULL;
+#ifdef ENABLE_STEP
     framePtr->text = NULL;
-    framePtr->fontPtr = NULL;
+    framePtr->tkfont = NULL;
     framePtr->justify = TK_JUSTIFY_CENTER;
     framePtr->normalFg = NULL;
     framePtr->textGC = None;
     framePtr->backGC = None;
     framePtr->numChars = 0;
 #endif
-    Tk_CreateEventHandler(new, ExposureMask|StructureNotifyMask|FocusChangeMask,
-	    FrameEventProc, (ClientData) framePtr);
+
+    /*
+     * Store backreference to frame widget in window structure.
+     */
+    TkSetClassProcs(new, NULL, (ClientData) framePtr);
+
+    mask = ExposureMask | StructureNotifyMask | FocusChangeMask;
+    if (toplevel) {
+        mask |= ActivateMask;
+    }
+    Tk_CreateEventHandler(new, mask, FrameEventProc, (ClientData) framePtr);
     if (ConfigureFrame(interp, framePtr, argc-2, argv+2, 0) != TCL_OK) {
 	goto error;
     }
+    if ((framePtr->isContainer)) {
+	if (framePtr->useThis == NULL) {
+	    TkpMakeContainer(framePtr->tkwin);
+	} else {
+	    Tcl_AppendResult(interp,"A window cannot have both the -use ",
+		    "and the -container option set.");
+	    return TCL_ERROR;
+	}
+    }
     if (toplevel) {
 	Tcl_DoWhenIdle(MapFrame, (ClientData) framePtr);
     }
@@ -461,7 +516,7 @@ FrameWidgetCmd(clientData, interp, argc, argv)
     char **argv;		/* Argument strings. */
 {
     register Frame *framePtr = (Frame *) clientData;
-    int result = TCL_OK;
+    int result;
     size_t length;
     int c, i;
 
@@ -494,8 +549,8 @@ FrameWidgetCmd(clientData, interp, argc, argv)
 		    (char *) framePtr, argv[2], framePtr->mask);
 	} else {
 	    /*
-	     * Don't allow the options -class, -newcmap, -screen,
-	     * or -visual to be changed.
+	     * Don't allow the options -class, -colormap, -container,
+	     * -newcmap, -screen, -use, or -visual to be changed.
 	     */
 
 	    for (i = 2; i < argc; i++) {
@@ -507,9 +562,15 @@ FrameWidgetCmd(clientData, interp, argc, argv)
 		if (((c == 'c') && (strncmp(argv[i], "-class", length) == 0)
 			&& (length >= 2))
 			|| ((c == 'c') && (framePtr->mask == TOPLEVEL)
-			&& (strncmp(argv[i], "-colormap", length) == 0))
+			&& (strncmp(argv[i], "-colormap", length) == 0)
+			&& (length >= 3))
+			|| ((c == 'c')
+			&& (strncmp(argv[i], "-container", length) == 0)
+			&& (length >= 3))
 			|| ((c == 's') && (framePtr->mask == TOPLEVEL)
 			&& (strncmp(argv[i], "-screen", length) == 0))
+			|| ((c == 'u') && (framePtr->mask == TOPLEVEL)
+			&& (strncmp(argv[i], "-use", length) == 0))
 			|| ((c == 'v') && (framePtr->mask == TOPLEVEL)
 			&& (strncmp(argv[i], "-visual", length) == 0))) {
 		    Tcl_AppendResult(interp, "can't modify ", argv[i],
@@ -556,11 +617,12 @@ DestroyFrame(memPtr)
 {
     register Frame *framePtr = (Frame *) memPtr;
 
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
     if (framePtr->textGC != None) {
 	Tk_FreeGC(framePtr->display, framePtr->textGC);
     }
 #endif
+
     Tk_FreeOptions(configSpecs, (char *) framePtr, framePtr->display,
 	    framePtr->mask);
     if (framePtr->colormap != None) {
@@ -599,23 +661,48 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
     char **argv;		/* Arguments. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
     XGCValues gcValues;
     GC newGC;
 #endif
+    char *oldMenuName;
+    
+    /*
+     * Need the old menubar name for the menu code to delete it.
+     */
+    
+    if (framePtr->menuName == NULL) {
+    	oldMenuName = NULL;
+    } else {
+    	oldMenuName = ckalloc(strlen(framePtr->menuName) + 1);
+    	strcpy(oldMenuName, framePtr->menuName);
+    }
+    
     if (Tk_ConfigureWidget(interp, framePtr->tkwin, configSpecs,
 	    argc, argv, (char *) framePtr, flags | framePtr->mask) != TCL_OK) {
 	return TCL_ERROR;
     }
+
+    if (((oldMenuName == NULL) && (framePtr->menuName != NULL))
+	    || ((oldMenuName != NULL) && (framePtr->menuName == NULL))
+	    || ((oldMenuName != NULL) && (framePtr->menuName != NULL)
+	    && strcmp(oldMenuName, framePtr->menuName) != 0)) {
+	TkSetWindowMenuBar(interp, framePtr->tkwin, oldMenuName,
+		framePtr->menuName);
+    }
+    
     if (framePtr->border != NULL) {
 	Tk_SetBackgroundFromBorder(framePtr->tkwin, framePtr->border);
+    } else {
+	Tk_SetWindowBackgroundPixmap(framePtr->tkwin, None);
     }
+
     if (framePtr->highlightWidth < 0) {
 	framePtr->highlightWidth = 0;
     }
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
     if (framePtr->mask & FRAME) {
-	gcValues.font = framePtr->fontPtr->fid;
+	gcValues.font = Tk_FontId(framePtr->tkfont);
 	gcValues.foreground = framePtr->normalFg->pixel;
 	gcValues.background = Tk_3DBorderColor(framePtr->border)->pixel;
 	newGC = Tk_GetGC(framePtr->tkwin,
@@ -627,8 +714,7 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
 	
 	framePtr->backGC = Tk_3DBorderGC(framePtr->tkwin, framePtr->border, 
 					 TK_3D_FLAT_GC);
-	framePtr->textHeight = framePtr->fontPtr->ascent
-	  		+ framePtr->fontPtr->descent;
+        Tk_GetFontMetrics(framePtr->tkfont, &(framePtr->fm));
 	framePtr->numChars = strlen(framePtr->text);
 	framePtr->textY = framePtr->highlightWidth;
     }
@@ -640,6 +726,10 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
 		framePtr->height);
     }
 
+    if (oldMenuName != NULL) {
+    	ckfree(oldMenuName);
+    }
+
     if (Tk_IsMapped(framePtr->tkwin)) {
 	if (!(framePtr->flags & REDRAW_PENDING)) {
 	    Tcl_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
@@ -673,36 +763,37 @@ DisplayFrame(clientData)
     register Frame *framePtr = (Frame *) clientData;
     register Tk_Window tkwin = framePtr->tkwin;
     GC gc;
-    
+
     framePtr->flags &= ~REDRAW_PENDING;
-    if ((framePtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
+    if ((framePtr->tkwin == NULL) || !Tk_IsMapped(tkwin)
+        || framePtr->isContainer) {
 	return;
     }
 
     if (framePtr->border != NULL) {
-#ifdef FULL_STEP	
+#ifdef ENABLE_STEP	
 	int dy;
 	
-	dy = (framePtr->numChars>0) ? (framePtr->textHeight-
+	dy = (framePtr->numChars>0) ? (framePtr->fm.linespace -
 				       framePtr->borderWidth)/2:0;
 	if ((framePtr->mask & FRAME) && framePtr->numChars > 0)
 	  XFillRectangle(framePtr->display, Tk_WindowId(tkwin),
 		framePtr->backGC, 0, 0, Tk_Width(tkwin),
-		framePtr->textHeight+framePtr->highlightWidth);
+		framePtr->fm.linespace+framePtr->highlightWidth);
 	Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
 		framePtr->border, framePtr->highlightWidth,
 		framePtr->highlightWidth+dy,
 		Tk_Width(tkwin) - 2*framePtr->highlightWidth,
 		Tk_Height(tkwin) - 2*framePtr->highlightWidth-dy,
 		framePtr->borderWidth, framePtr->relief);	
-#else	
+#else
 	Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
 		framePtr->border, framePtr->highlightWidth,
 		framePtr->highlightWidth,
 		Tk_Width(tkwin) - 2*framePtr->highlightWidth,
 		Tk_Height(tkwin) - 2*framePtr->highlightWidth,
-		framePtr->borderWidth, framePtr->relief);	
-#endif	
+		framePtr->borderWidth, framePtr->relief);
+#endif
     }
     if (framePtr->highlightWidth != 0) {
 	if (framePtr->flags & GOT_FOCUS) {
@@ -715,15 +806,15 @@ DisplayFrame(clientData)
 	Tk_DrawFocusHighlight(tkwin, gc, framePtr->highlightWidth,
 		Tk_WindowId(tkwin));
     }
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
     if ((framePtr->mask & FRAME) && framePtr->numChars > 0) {
 	int x, frameWidth, bx;
 
 	frameWidth = Tk_Width(tkwin)-2*framePtr->highlightWidth -
 	  	2*framePtr->borderWidth;
-	TkMeasureChars(framePtr->fontPtr, framePtr->text,
-		framePtr->numChars, 0, frameWidth,
-		0, TK_PARTIAL_OK|TK_AT_LEAST_ONE, &(framePtr->textWidth));
+	Tk_MeasureChars(framePtr->tkfont, framePtr->text,
+		framePtr->numChars, 0,
+		TK_PARTIAL_OK|TK_AT_LEAST_ONE, &(framePtr->textWidth));
 	x = framePtr->borderWidth + framePtr->highlightWidth;
 
 	if (framePtr->justify == TK_JUSTIFY_LEFT) {
@@ -731,16 +822,16 @@ DisplayFrame(clientData)
 	} else if (framePtr->justify == TK_JUSTIFY_CENTER) {
 	    bx = (frameWidth - framePtr->textWidth - x)/2;
 	} else {
-	    bx = x + (frameWidth - framePtr->textWidth);
+	    bx = frameWidth - framePtr->textWidth - framePtr->borderWidth;
 	}
 	XFillRectangle(framePtr->display, Tk_WindowId(tkwin),framePtr->backGC,
-		       bx, framePtr->textY,framePtr->textWidth+2*x,
-		       framePtr->textHeight);
-	TkDisplayText(framePtr->display, Tk_WindowId(tkwin), 
-		framePtr->fontPtr, framePtr->text, framePtr->numChars, x, framePtr->textY,
-		frameWidth, framePtr->justify, -1, framePtr->textGC);
+		       bx, framePtr->textY, framePtr->textWidth+2*x,
+		       framePtr->fm.linespace);
+	Tk_DrawChars(framePtr->display, Tk_WindowId(tkwin), framePtr->textGC, 
+		framePtr->tkfont, framePtr->text, framePtr->numChars,
+                bx+x, framePtr->textY + framePtr->fm.ascent);
     }
-#endif    
+#endif   
 }
 
 /*
@@ -773,10 +864,29 @@ FrameEventProc(clientData, eventPtr)
 	    || (eventPtr->type == ConfigureNotify)) {
 	goto redraw;
     } else if (eventPtr->type == DestroyNotify) {
+	if (framePtr->menuName != NULL) {
+	    TkSetWindowMenuBar(framePtr->interp, framePtr->tkwin,
+		    framePtr->menuName, NULL);
+	    ckfree(framePtr->menuName);
+	    framePtr->menuName = NULL;
+	}
 	if (framePtr->tkwin != NULL) {
+
+	    /*
+	     * If this window is a container, then this event could be
+	     * coming from the embedded application, in which case
+	     * Tk_DestroyWindow hasn't been called yet.  When Tk_DestroyWindow
+	     * is called later, then another destroy event will be generated.
+	     * We need to be sure we ignore the second event, since the frame
+	     * could be gone by then.  To do so, delete the event handler
+	     * explicitly (normally it's done implicitly by Tk_DestroyWindow).
+	     */
+    
+	    Tk_DeleteEventHandler(framePtr->tkwin,
+		    ExposureMask|StructureNotifyMask|FocusChangeMask,
+		    FrameEventProc, (ClientData) framePtr);
 	    framePtr->tkwin = NULL;
-	    Tcl_DeleteCommand(framePtr->interp,
-		    Tcl_GetCommandName(framePtr->interp, framePtr->widgetCmd));
+            Tcl_DeleteCommandFromToken(framePtr->interp, framePtr->widgetCmd);
 	}
 	if (framePtr->flags & REDRAW_PENDING) {
 	    Tcl_CancelIdleCall(DisplayFrame, (ClientData) framePtr);
@@ -797,6 +907,9 @@ FrameEventProc(clientData, eventPtr)
 		goto redraw;
 	    }
 	}
+    } else if (eventPtr->type == ActivateNotify) {
+    	TkpSetMainMenubar(framePtr->interp, framePtr->tkwin,
+    		framePtr->menuName);
     }
     return;
 
@@ -832,12 +945,20 @@ FrameCmdDeletedProc(clientData)
     Frame *framePtr = (Frame *) clientData;
     Tk_Window tkwin = framePtr->tkwin;
 
+    if (framePtr->menuName != NULL) {
+	TkSetWindowMenuBar(framePtr->interp, framePtr->tkwin,
+		framePtr->menuName, NULL);
+	ckfree(framePtr->menuName);
+	framePtr->menuName = NULL;
+    }
+
     /*
      * This procedure could be invoked either because the window was
      * destroyed and the command was then deleted (in which case tkwin
      * is NULL) or because the command was deleted, and then this procedure
      * destroys the widget.
      */
+
     if (tkwin != NULL) {
 	framePtr->tkwin = NULL;
 	Tk_DestroyWindow(tkwin);
@@ -893,3 +1014,37 @@ MapFrame(clientData)
     Tk_MapWindow(framePtr->tkwin);
     Tcl_Release((ClientData) framePtr);
 }
+
+/*
+ *--------------------------------------------------------------
+ *
+ * TkInstallFrameMenu --
+ *
+ *	This function is needed when a Windows HWND is created
+ *	and a menubar has been set to the window with a system
+ *	menu. It notifies the menu package so that the system
+ *	menu can be rebuilt.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The system menu (if any) is created for the menubar
+ *	associated with this frame.
+ *
+ *--------------------------------------------------------------
+ */
+
+void
+TkInstallFrameMenu(tkwin)
+    Tk_Window tkwin;		/* The window that was just created. */
+{
+    TkWindow *winPtr = (TkWindow *) tkwin;
+
+    if (winPtr->mainPtr != NULL) {
+	Frame *framePtr;
+	framePtr = (Frame*) winPtr->instanceData;
+	TkpMenuNotifyToplevelCreate(winPtr->mainPtr->interp, 
+		framePtr->menuName);
+    }
+}
