@@ -6,12 +6,16 @@
  *	one per line, and provides scrolling and selection.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkListbox.c 1.109 96/05/17 16:26:55
+ * SCCS: @(#) tkListbox.c 1.120 97/10/29 13:06:59
+ */
+/*
+ * TkSTEP modifications Copyright (c) 1997 by Oliver Graf <ograf@fga.de>
+ * - surgery performed and implanted into Tk8.0 by Steve Murray
  */
 
 #include "tkPort.h"
@@ -24,23 +28,23 @@
  */
 
 typedef struct Element {
-  int textLength;	/* # non-NULL characters in text. */
-  int lBearing;		/* Distance from first character's
-			 * origin to left edge of character. */
-  int pixelWidth;	/* Total width of element in pixels (including
-			 * left bearing and right bearing). */
-  int selected;		/* 1 means this item is selected, 0 means
-			 * it isn't. */
-#ifdef FULL_STEP
-  int arrow;            /* append a *Step arrow to the entry */
+    int textLength;		/* # non-NULL characters in text. */
+    int lBearing;		/* Distance from first character's
+				 * origin to left edge of character. */
+    int pixelWidth;		/* Total width of element in pixels (including
+				 * left bearing and right bearing). */
+    int selected;		/* 1 means this item is selected, 0 means
+				 * it isn't. */
+#ifdef ENABLE_STEP
+    int arrow;                  /* append a N*XTSTEP arrow to the entry. */
 #endif
-  struct Element *nextPtr;	/* Next in list of all elements of this
+    struct Element *nextPtr;	/* Next in list of all elements of this
 				 * listbox, or NULL for last element. */
-  char text[4];		/* Characters of this element, NULL-
-			 * terminated.  The actual space allocated
-			 * here will be as large as needed (> 4,
-			 * most likely).  Must be the last field
-			 * of the record. */
+    char text[4];		/* Characters of this element, NULL-
+				 * terminated.  The actual space allocated
+				 * here will be as large as needed (> 4,
+				 * most likely).  Must be the last field
+				 * of the record. */
 } Element;
 
 #define ElementSize(stringLength) \
@@ -87,7 +91,7 @@ typedef struct {
 				 * Indicates how much interior stuff must
 				 * be offset from outside edges to leave
 				 * room for borders. */
-    XFontStruct *fontPtr;	/* Information about text font, or NULL. */
+    Tk_Font tkfont;		/* Information about text font, or NULL. */
     XColor *fgColorPtr;		/* Text color in normal mode. */
     GC textGC;			/* For drawing normal text. */
     Tk_3DBorder selBorder;	/* Borders and backgrounds for selected
@@ -218,7 +222,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *) NULL,
 	(char *) NULL, 0, 0},
     {TK_CONFIG_FONT, "-font", "font", "Font",
-	DEF_LISTBOX_FONT, Tk_Offset(Listbox, fontPtr), 0},
+	DEF_LISTBOX_FONT, Tk_Offset(Listbox, tkfont), 0},
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
 	DEF_LISTBOX_FG, Tk_Offset(Listbox, fgColorPtr), 0},
     {TK_CONFIG_INT, "-height", "height", "Height",
@@ -282,7 +286,7 @@ static void		DeleteEls _ANSI_ARGS_((Listbox *listPtr, int first,
 static void		DestroyListbox _ANSI_ARGS_((char *memPtr));
 static void		DisplayListbox _ANSI_ARGS_((ClientData clientData));
 static int		GetListboxIndex _ANSI_ARGS_((Tcl_Interp *interp,
-			    Listbox *listPtr, char *string, int numElsOK,
+			    Listbox *listPtr, char *string, int endIsSize,
 			    int *indexPtr));
 static void		InsertEls _ANSI_ARGS_((Listbox *listPtr, int index,
 			    int argc, char **argv));
@@ -307,9 +311,22 @@ static void		ListboxUpdateHScrollbar _ANSI_ARGS_((Listbox *listPtr));
 static void		ListboxUpdateVScrollbar _ANSI_ARGS_((Listbox *listPtr));
 static int		ListboxWidgetCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
+static void		ListboxWorldChanged _ANSI_ARGS_((
+			    ClientData instanceData));
 static int		NearestListboxElement _ANSI_ARGS_((Listbox *listPtr,
 			    int y));
 
+/*
+ * The structure below defines button class behavior by means of procedures
+ * that can be invoked from generic window code.
+ */
+
+static TkClassProcs listboxClass = {
+    NULL,			/* createProc. */
+    ListboxWorldChanged,	/* geometryProc. */
+    NULL			/* modalProc. */
+};
+
 
 /*
  *--------------------------------------------------------------
@@ -375,7 +392,7 @@ Tk_ListboxCmd(clientData, interp, argc, argv)
     listPtr->highlightBgColorPtr = NULL;
     listPtr->highlightColorPtr = NULL;
     listPtr->inset = 0;
-    listPtr->fontPtr = NULL;
+    listPtr->tkfont = NULL;
     listPtr->fgColorPtr = NULL;
     listPtr->textGC = None;
     listPtr->selBorder = NULL;
@@ -390,7 +407,7 @@ Tk_ListboxCmd(clientData, interp, argc, argv)
     listPtr->partialLine = 0;
     listPtr->setGrid = 0;
     listPtr->maxWidth = 0;
-    listPtr->xScrollUnit = 0;
+    listPtr->xScrollUnit = 1;
     listPtr->xOffset = 0;
     listPtr->selectMode = NULL;
     listPtr->numSelected = 0;
@@ -408,6 +425,7 @@ Tk_ListboxCmd(clientData, interp, argc, argv)
     listPtr->flags = 0;
 
     Tk_SetClass(listPtr->tkwin, "Listbox");
+    TkSetClassProcs(listPtr->tkwin, &listboxClass, (ClientData) listPtr);
     Tk_CreateEventHandler(listPtr->tkwin,
 	    ExposureMask|StructureNotifyMask|FocusChangeMask,
 	    ListboxEventProc, (ClientData) listPtr);
@@ -454,6 +472,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
     int result = TCL_OK;
     size_t length;
     int c;
+    Tk_FontMetrics fm;
 
     if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -473,10 +492,15 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	    goto error;
 	}
 	ListboxRedrawRange(listPtr, listPtr->active, listPtr->active);
-	if (GetListboxIndex(interp, listPtr, argv[2], 0, &index)
-		!= TCL_OK) {
+	if (GetListboxIndex(interp, listPtr, argv[2], 0, &index) != TCL_OK) {
 	    goto error;
 	}
+	if (index >= listPtr->numElements) {
+	    index = listPtr->numElements-1;
+	}
+	if (index < 0) {
+	    index = 0;
+	}
 	listPtr->active = index;
 	ListboxRedrawRange(listPtr, listPtr->active, listPtr->active);
     } else if ((c == 'b') && (strncmp(argv[1], "bbox", length) == 0)) {
@@ -491,6 +515,9 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	if (GetListboxIndex(interp, listPtr, argv[2], 0, &index) != TCL_OK) {
 	    goto error;
 	}
+	if ((index >= listPtr->numElements) || (index < 0)) {
+	    goto done;
+	}
 	for (i = 0, elPtr = listPtr->firstPtr; i < index;
 		i++, elPtr = elPtr->nextPtr) {
 	    /* Empty loop body. */
@@ -498,11 +525,12 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	if ((index >= listPtr->topIndex) && (index < listPtr->numElements)
 		    && (index < (listPtr->topIndex + listPtr->fullLines
 		    + listPtr->partialLine))) {
-	    x = listPtr->inset - listPtr->xOffset;
+	    x = listPtr->inset + listPtr->selBorderWidth - listPtr->xOffset;
 	    y = ((index - listPtr->topIndex)*listPtr->lineHeight)
 		    + listPtr->inset + listPtr->selBorderWidth;
+	    Tk_GetFontMetrics(listPtr->tkfont, &fm);
 	    sprintf(interp->result, "%d %d %d %d", x, y, elPtr->pixelWidth,
-		    listPtr->fontPtr->ascent + listPtr->fontPtr->descent);
+		    fm.linespace);
 	}
     } else if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
 	    && (length >= 2)) {
@@ -562,14 +590,20 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	if (GetListboxIndex(interp, listPtr, argv[2], 0, &first) != TCL_OK) {
 	    goto error;
 	}
-	if (argc == 3) {
-	    last = first;
-	} else {
-	    if (GetListboxIndex(interp, listPtr, argv[3], 0, &last) != TCL_OK) {
-		goto error;
+	if (first < listPtr->numElements) {
+	    if (argc == 3) {
+		last = first;
+	    } else {
+		if (GetListboxIndex(interp, listPtr, argv[3], 0,
+			&last) != TCL_OK) {
+		    goto error;
+		}
+		if (last >= listPtr->numElements) {
+		    last = listPtr->numElements-1;
+		}
 	    }
+	    DeleteEls(listPtr, first, last);
 	}
-	DeleteEls(listPtr, first, last);
     } else if ((c == 'g') && (strncmp(argv[1], "get", length) == 0)) {
 	int first, last, i;
 	Element *elPtr;
@@ -586,13 +620,22 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		0, &last) != TCL_OK)) {
 	    goto error;
 	}
+	if (first >= listPtr->numElements) {
+	    goto done;
+	}
+	if (last >= listPtr->numElements) {
+	    last = listPtr->numElements-1;
+	}
+
 	for (elPtr = listPtr->firstPtr, i = 0; i < first;
 		i++, elPtr = elPtr->nextPtr) {
 	    /* Empty loop body. */
 	}
 	if (elPtr != NULL) {
 	    if (argc == 3) {
-		interp->result = elPtr->text;
+		if (first >= 0) {
+		    interp->result = elPtr->text;
+		}
 	    } else {
 		for (  ; i <= last; i++, elPtr = elPtr->nextPtr) {
 		    Tcl_AppendElement(interp, elPtr->text);
@@ -681,6 +724,12 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	if (GetListboxIndex(interp, listPtr, argv[2], 0, &index) != TCL_OK) {
 	    goto error;
 	}
+	if (index >= listPtr->numElements) {
+	    index = listPtr->numElements-1;
+	}
+	if (index < 0) {
+	    index = 0;
+	}
 	diff = listPtr->topIndex-index;
 	if (diff > 0) {
 	    if (diff <= (listPtr->fullLines/3)) {
@@ -727,6 +776,12 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		    argv[0], " selection anchor index\"", (char *) NULL);
 		goto error;
 	    }
+	    if (first >= listPtr->numElements) {
+		first = listPtr->numElements-1;
+	    }
+	    if (first < 0) {
+		first = 0;
+	    }
 	    listPtr->selectAnchor = first;
 	} else if ((c == 'c') && (strncmp(argv[2], "clear", length) == 0)) {
 	    ListboxSelect(listPtr, first, last, 0);
@@ -739,11 +794,15 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 			argv[0], " selection includes index\"", (char *) NULL);
 		goto error;
 	    }
+	    if ((first < 0) || (first >= listPtr->numElements)) {
+		interp->result = "0";
+		goto done;
+	    }
 	    for (elPtr = listPtr->firstPtr, i = 0; i < first;
 		    i++, elPtr = elPtr->nextPtr) {
 		/* Empty loop body. */
 	    }
-	    if ((elPtr != NULL) && (elPtr->selected)) {
+	    if (elPtr->selected) {
 		interp->result = "1";
 	    } else {
 		interp->result = "0";
@@ -794,7 +853,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		case TK_SCROLL_ERROR:
 		    goto error;
 		case TK_SCROLL_MOVETO:
-		    offset = fraction*listPtr->maxWidth + 0.5;
+		    offset = (int) (fraction*listPtr->maxWidth + 0.5);
 		    break;
 		case TK_SCROLL_PAGES:
 		    windowUnits = windowWidth/listPtr->xScrollUnit;
@@ -839,7 +898,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		case TK_SCROLL_ERROR:
 		    goto error;
 		case TK_SCROLL_MOVETO:
-		    index = listPtr->numElements*fraction + 0.5;
+		    index = (int) (listPtr->numElements*fraction + 0.5);
 		    break;
 		case TK_SCROLL_PAGES:
 		    if (listPtr->fullLines > 2) {
@@ -863,6 +922,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		"xview, or yview", (char *) NULL);
 	goto error;
     }
+    done:
     Tcl_Release((ClientData) listPtr);
     return result;
 
@@ -952,8 +1012,6 @@ ConfigureListbox(interp, listPtr, argc, argv, flags)
     char **argv;		/* Arguments. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
-    XGCValues gcValues;
-    GC new;
     int oldExport;
 
     oldExport = listPtr->exportSelection;
@@ -974,24 +1032,6 @@ ConfigureListbox(interp, listPtr, argc, argv, flags)
     }
     listPtr->inset = listPtr->highlightWidth + listPtr->borderWidth;
 
-    gcValues.foreground = listPtr->fgColorPtr->pixel;
-    gcValues.font = listPtr->fontPtr->fid;
-    gcValues.graphics_exposures = False;
-    new = Tk_GetGC(listPtr->tkwin, GCForeground|GCFont|GCGraphicsExposures,
-	    &gcValues);
-    if (listPtr->textGC != None) {
-	Tk_FreeGC(listPtr->display, listPtr->textGC);
-    }
-    listPtr->textGC = new;
-
-    gcValues.foreground = listPtr->selFgColorPtr->pixel;
-    gcValues.font = listPtr->fontPtr->fid;
-    new = Tk_GetGC(listPtr->tkwin, GCForeground|GCFont, &gcValues);
-    if (listPtr->selTextGC != None) {
-	Tk_FreeGC(listPtr->display, listPtr->selTextGC);
-    }
-    listPtr->selTextGC = new;
-
     /*
      * Claim the selection if we've suddenly started exporting it and
      * there is a selection to export.
@@ -1003,6 +1043,58 @@ ConfigureListbox(interp, listPtr, argc, argv, flags)
 		(ClientData) listPtr);
     }
 
+    ListboxWorldChanged((ClientData) listPtr);
+    return TCL_OK;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * ListboxWorldChanged --
+ *
+ *      This procedure is called when the world has changed in some
+ *      way and the widget needs to recompute all its graphics contexts
+ *	and determine its new geometry.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Listbox will be relayed out and redisplayed.
+ *
+ *---------------------------------------------------------------------------
+ */
+ 
+static void
+ListboxWorldChanged(instanceData)
+    ClientData instanceData;	/* Information about widget. */
+{
+    XGCValues gcValues;
+    GC gc;
+    unsigned long mask;
+    Listbox *listPtr;
+
+    listPtr = (Listbox *) instanceData;
+
+    gcValues.foreground = listPtr->fgColorPtr->pixel;
+    gcValues.font = Tk_FontId(listPtr->tkfont);
+    gcValues.graphics_exposures = False;
+    mask = GCForeground | GCFont | GCGraphicsExposures;
+    gc = Tk_GetGC(listPtr->tkwin, mask, &gcValues);
+    if (listPtr->textGC != None) {
+	Tk_FreeGC(listPtr->display, listPtr->textGC);
+    }
+    listPtr->textGC = gc;
+
+    gcValues.foreground = listPtr->selFgColorPtr->pixel;
+    gcValues.font = Tk_FontId(listPtr->tkfont);
+    mask = GCForeground | GCFont;
+    gc = Tk_GetGC(listPtr->tkwin, mask, &gcValues);
+    if (listPtr->selTextGC != None) {
+	Tk_FreeGC(listPtr->display, listPtr->selTextGC);
+    }
+    listPtr->selTextGC = gc;
+
     /*
      * Register the desired geometry for the window and arrange for
      * the window to be redisplayed.
@@ -1011,7 +1103,6 @@ ConfigureListbox(interp, listPtr, argc, argv, flags)
     ListboxComputeGeometry(listPtr, 1, 1, 1);
     listPtr->flags |= UPDATE_V_SCROLLBAR|UPDATE_H_SCROLLBAR;
     ListboxRedrawRange(listPtr, 0, listPtr->numElements-1);
-    return TCL_OK;
 }
 
 /*
@@ -1039,6 +1130,7 @@ DisplayListbox(clientData)
     register Element *elPtr;
     GC gc;
     int i, limit, x, y, width, prevSelected;
+    Tk_FontMetrics fm;
     int left, right;			/* Non-zero values here indicate
 					 * that the left or right edge of
 					 * the listbox is off-screen. */
@@ -1141,20 +1233,21 @@ DisplayListbox(clientData)
 			TK_RELIEF_RAISED);
 	    }
 	}
-	y += listPtr->fontPtr->ascent + listPtr->selBorderWidth;
+	Tk_GetFontMetrics(listPtr->tkfont, &fm);
+	y += fm.ascent + listPtr->selBorderWidth;
 	x = listPtr->inset + listPtr->selBorderWidth - elPtr->lBearing
 		- listPtr->xOffset;
-	XDrawString(listPtr->display, pixmap, gc, x, y,
-		elPtr->text, elPtr->textLength);
+	Tk_DrawChars(listPtr->display, pixmap, gc, listPtr->tkfont,
+		elPtr->text, elPtr->textLength, x, y);
 
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
 	if (elPtr->arrow)
 	  {
 	    int tx, ty;
 	    GC lightGC, darkGC, dark2GC;
 	    
 	    tx = listPtr->width*listPtr->xScrollUnit + listPtr->inset - 12;
-	    ty = y - listPtr->fontPtr->ascent/2 - 3;
+	    ty = y - fm.ascent/2 - 3;
 	    
 	    lightGC = Tk_3DBorderGC(listPtr->tkwin,
 		elPtr->selected?listPtr->selBorder:listPtr->normalBorder,
@@ -1167,9 +1260,9 @@ DisplayListbox(clientData)
 		TK_3D_DARK2_GC);
 
 	    XDrawLine(listPtr->display, pixmap, darkGC,
-	    	      tx, ty, tx+7, ty+3);
+	    	      tx, ty-1, tx+7, ty+3);
 	    XDrawLine(listPtr->display, pixmap, lightGC,
-	    	      tx, ty+6, tx+7, ty+3);
+	    	      tx, ty+7, tx+7, ty+3);
 	    XDrawLine(listPtr->display, pixmap, dark2GC,
 	    	      tx, ty, tx, ty+6);
 	  }
@@ -1180,11 +1273,8 @@ DisplayListbox(clientData)
 	 */
 
 	if ((i == listPtr->active) && (listPtr->flags & GOT_FOCUS)) {
-	    XFillRectangle(listPtr->display, pixmap, gc,
-		    listPtr->inset + listPtr->selBorderWidth
-			- listPtr->xOffset,
-		    y + listPtr->fontPtr->descent - 1,
-		    (unsigned) elPtr->pixelWidth, 1);
+	    Tk_UnderlineChars(listPtr->display, pixmap, gc, listPtr->tkfont,
+		    elPtr->text, x, y, 0, elPtr->textLength);
 	}
     }
 
@@ -1250,18 +1340,20 @@ ListboxComputeGeometry(listPtr, fontChanged, maxIsStale, updateGrid)
 				 * the window. */
 {
     register Element *elPtr;
-    int dummy, fontHeight, width, height, pixelWidth, pixelHeight;
-    XCharStruct bbox;
+    int width, height, pixelWidth, pixelHeight;
+    Tk_FontMetrics fm;
 
     if (fontChanged  || maxIsStale) {
-	listPtr->xScrollUnit = XTextWidth(listPtr->fontPtr, "0", 1);
+	listPtr->xScrollUnit = Tk_TextWidth(listPtr->tkfont, "0", 1);
+	if (listPtr->xScrollUnit == 0) {
+	    listPtr->xScrollUnit = 1;
+	}
 	listPtr->maxWidth = 0;
 	for (elPtr = listPtr->firstPtr; elPtr != NULL; elPtr = elPtr->nextPtr) {
 	    if (fontChanged) {
-		XTextExtents(listPtr->fontPtr, elPtr->text, elPtr->textLength,
-			&dummy, &dummy, &dummy, &bbox);
-		elPtr->lBearing = bbox.lbearing;
-		elPtr->pixelWidth = bbox.rbearing - bbox.lbearing;
+		elPtr->pixelWidth = Tk_TextWidth(listPtr->tkfont,
+			elPtr->text, elPtr->textLength);
+		elPtr->lBearing = 0;
 	    }
 	    if (elPtr->pixelWidth > listPtr->maxWidth) {
 		listPtr->maxWidth = elPtr->pixelWidth;
@@ -1269,11 +1361,11 @@ ListboxComputeGeometry(listPtr, fontChanged, maxIsStale, updateGrid)
 	}
     }
 
-    fontHeight = listPtr->fontPtr->ascent + listPtr->fontPtr->descent;
-    listPtr->lineHeight = fontHeight + 1 + 2*listPtr->selBorderWidth;
-#ifdef FULL_STEP
+    Tk_GetFontMetrics(listPtr->tkfont, &fm);
+    listPtr->lineHeight = fm.linespace + 1 + 2*listPtr->selBorderWidth;
+#ifdef ENABLE_STEP
     if (listPtr->lineHeight < 11)
-      listPtr->lineHeight = 11;
+        listPtr->lineHeight = 11;
 #endif
     width = listPtr->width;
     if (width <= 0) {
@@ -1332,11 +1424,10 @@ InsertEls(listPtr, index, argc, argv)
     char **argv;		/* New elements (one per entry). */
 {
     register Element *prevPtr, *newPtr;
-    int length, dummy, i, oldMaxWidth;
-#ifdef FULL_STEP
+    int length, i, oldMaxWidth;
+#ifdef ENABLE_STEP
     int arrow;
 #endif
-    XCharStruct bbox;
 
     /*
      * Find the element before which the new ones will be inserted.
@@ -1366,27 +1457,24 @@ InsertEls(listPtr, index, argc, argv)
     oldMaxWidth = listPtr->maxWidth;
     for (i = argc ; i > 0; i--, argv++, prevPtr = newPtr) {
       length = strlen(*argv);
-#ifdef FULL_STEP
-      if (strcmp(*argv+length-3," ->")==0)
-	{
-	  arrow = 1;
-	  length -= 3;
-	  (*argv)[length] = 0;
-	}
-      else
-	arrow = 0;
+#ifdef ENABLE_STEP
+      if (strcmp(*argv+length-3," ->") == 0) {
+          arrow = 1;
+          length -= 3;
+          (*argv)[length] = 0;
+      } else
+        arrow = 0;
 #endif
 	newPtr = (Element *) ckalloc(ElementSize(length));
 	newPtr->textLength = length;
 	strcpy(newPtr->text, *argv);
-	XTextExtents(listPtr->fontPtr, newPtr->text, newPtr->textLength,
-		&dummy, &dummy, &dummy, &bbox);
-	newPtr->lBearing = bbox.lbearing;
-	newPtr->pixelWidth = bbox.rbearing - bbox.lbearing;
-#ifdef FULL_STEP
-	newPtr->arrow=arrow;
-	if (newPtr->arrow)
-	  newPtr->pixelWidth += 16;
+	newPtr->pixelWidth = Tk_TextWidth(listPtr->tkfont, newPtr->text,
+		newPtr->textLength);
+	newPtr->lBearing = 0;
+#ifdef ENABLE_STEP
+        newPtr->arrow = arrow;
+        if (newPtr->arrow)
+            newPtr->pixelWidth += 16;
 #endif
 	if (newPtr->pixelWidth > listPtr->maxWidth) {
 	    listPtr->maxWidth = newPtr->pixelWidth;
@@ -1592,8 +1680,7 @@ ListboxEventProc(clientData, eventPtr)
 		Tk_UnsetGrid(listPtr->tkwin);
 	    }
 	    listPtr->tkwin = NULL;
-	    Tcl_DeleteCommand(listPtr->interp,
-		    Tcl_GetCommandName(listPtr->interp, listPtr->widgetCmd));
+	    Tcl_DeleteCommandFromToken(listPtr->interp, listPtr->widgetCmd);
 	}
 	if (listPtr->flags & REDRAW_PENDING) {
 	    Tcl_CancelIdleCall(DisplayListbox, (ClientData) listPtr);
@@ -1695,15 +1782,15 @@ ListboxCmdDeletedProc(clientData)
  */
 
 static int
-GetListboxIndex(interp, listPtr, string, numElsOK, indexPtr)
+GetListboxIndex(interp, listPtr, string, endIsSize, indexPtr)
     Tcl_Interp *interp;		/* For error messages. */
     Listbox *listPtr;		/* Listbox for which the index is being
 				 * specified. */
     char *string;		/* Specifies an element in the listbox. */
-    int numElsOK;		/* 0 means the return value must be less
-				 * less than the number of entries in
-				 * the listbox;  1 means it may also be
-				 * equal to the number of entries. */
+    int endIsSize;		/* If 1, "end" refers to the number of
+				 * entries in the listbox.  If 0, "end"
+				 * refers to 1 less than the number of
+				 * entries. */
     int *indexPtr;		/* Where to store converted index. */
 {
     int c;
@@ -1718,13 +1805,17 @@ GetListboxIndex(interp, listPtr, string, numElsOK, indexPtr)
 	    && (length >= 2)) {
 	*indexPtr = listPtr->selectAnchor;
     } else if ((c == 'e') && (strncmp(string, "end", length) == 0)) {
-	*indexPtr = listPtr->numElements;
+	if (endIsSize) {
+	    *indexPtr = listPtr->numElements;
+	} else {
+	    *indexPtr = listPtr->numElements - 1;
+	}
     } else if (c == '@') {
-	int x, y;
+	int y;
 	char *p, *end;
 
 	p = string+1;
-	x = strtol(p, &end, 0);
+	strtol(p, &end, 0);
 	if ((end == p) || (*end != ',')) {
 	    goto badIndex;
 	}
@@ -1740,16 +1831,6 @@ GetListboxIndex(interp, listPtr, string, numElsOK, indexPtr)
 	    goto badIndex;
 	}
     }
-    if (numElsOK) {
-	if (*indexPtr > listPtr->numElements) {
-	    *indexPtr = listPtr->numElements;
-	}
-    } else if (*indexPtr >= listPtr->numElements) {
-	*indexPtr = listPtr->numElements-1;
-    }
-    if (*indexPtr < 0) {
-	*indexPtr = 0;
-    }
     return TCL_OK;
 
     badIndex:
@@ -1831,16 +1912,16 @@ ChangeListboxOffset(listPtr, offset)
      * round it off to an even multiple of xScrollUnit.
      */
 
-    maxOffset = listPtr->maxWidth + (listPtr->xScrollUnit-1)
-	    - (Tk_Width(listPtr->tkwin) - 2*listPtr->inset
-	    - 2*listPtr->selBorderWidth - listPtr->xScrollUnit);
+    maxOffset = listPtr->maxWidth - (Tk_Width(listPtr->tkwin) -
+	    2*listPtr->inset - 2*listPtr->selBorderWidth)
+	    + listPtr->xScrollUnit - 1;
     if (offset > maxOffset) {
 	offset = maxOffset;
     }
     if (offset < 0) {
 	offset = 0;
     }
-    offset -= offset%listPtr->xScrollUnit;
+    offset -= offset % listPtr->xScrollUnit;
     if (offset != listPtr->xOffset) {
 	listPtr->xOffset = offset;
 	listPtr->flags |= UPDATE_H_SCROLLBAR;
@@ -1876,7 +1957,7 @@ ListboxScanTo(listPtr, x, y)
     int newTopIndex, newOffset, maxIndex, maxOffset;
 
     maxIndex = listPtr->numElements - listPtr->fullLines;
-    maxOffset = listPtr->maxWidth + (listPtr->xScrollUnit-1)
+    maxOffset = listPtr->maxWidth + (listPtr->xScrollUnit - 1)
 	    - (Tk_Width(listPtr->tkwin) - 2*listPtr->inset
 	    - 2*listPtr->selBorderWidth - listPtr->xScrollUnit);
 
@@ -1989,7 +2070,7 @@ ListboxSelect(listPtr, first, last, select)
     int select;				/* 1 means select items, 0 means
 					 * deselect them. */
 {
-    int i, firstRedisplay, lastRedisplay, increment, oldCount;
+    int i, firstRedisplay, increment, oldCount;
     Element *elPtr;
 
     if (last < first) {
@@ -1997,9 +2078,15 @@ ListboxSelect(listPtr, first, last, select)
 	first = last;
 	last = i;
     }
-    if (first >= listPtr->numElements) {
+    if ((last < 0) || (first >= listPtr->numElements)) {
 	return;
     }
+    if (first < 0) {
+	first = 0;
+    }
+    if (last >= listPtr->numElements) {
+	last = listPtr->numElements - 1;
+    }
     oldCount = listPtr->numSelected;
     firstRedisplay = -1;
     increment = select ? 1 : -1;
@@ -2016,7 +2103,6 @@ ListboxSelect(listPtr, first, last, select)
 	if (firstRedisplay < 0) {
 	    firstRedisplay = i;
 	}
-	lastRedisplay = i;
     }
     if (firstRedisplay >= 0) {
 	ListboxRedrawRange(listPtr, first, last);
