@@ -13,7 +13,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkRectOval.c 1.39 96/03/02 17:28:06
+ * SCCS: @(#) tkRectOval.c 1.40 96/05/03 10:52:21
  */
 
 #include <stdio.h>
@@ -32,7 +32,7 @@ typedef struct RectOvalItem  {
 				 * or oval (x1, y1, x2, y2).  Item includes
 				 * x1 and x2 but not y1 and y2. */
   int width;			/* Width of outline. */
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
   int rounded;                  /* Rounded corners for rectangle */
 #endif
   XColor *outlineColor;         /* Color for outline. */
@@ -55,7 +55,7 @@ static Tk_ConfigSpec configSpecs[] = {
 	(char *) NULL, Tk_Offset(RectOvalItem, fillColor), TK_CONFIG_NULL_OK},
     {TK_CONFIG_COLOR, "-outline", (char *) NULL, (char *) NULL,
 	"black", Tk_Offset(RectOvalItem, outlineColor), TK_CONFIG_NULL_OK},
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
     {TK_CONFIG_PIXELS, "-rounded", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(RectOvalItem, rounded), TK_CONFIG_NULL_OK},
 #endif
@@ -201,7 +201,7 @@ CreateRectOval(interp, canvas, itemPtr, argc, argv)
      */
 
     rectOvalPtr->width = 1;
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
     rectOvalPtr->rounded = 0;
 #endif
     rectOvalPtr->outlineColor = NULL;
@@ -480,23 +480,23 @@ ComputeRectOvalBbox(canvas, rectOvalPtr)
      * greater than the lower ones.
      */
 
-    tmp = (rectOvalPtr->bbox[0] >= 0) ? rectOvalPtr->bbox[0] + .5
-	    : rectOvalPtr->bbox[0] - .5;
+    tmp = (int) ((rectOvalPtr->bbox[0] >= 0) ? rectOvalPtr->bbox[0] + .5
+	    : rectOvalPtr->bbox[0] - .5);
     rectOvalPtr->header.x1 = tmp - bloat;
-    tmp = (rectOvalPtr->bbox[1] >= 0) ? rectOvalPtr->bbox[1] + .5
-	    : rectOvalPtr->bbox[1] - .5;
+    tmp = (int) ((rectOvalPtr->bbox[1] >= 0) ? rectOvalPtr->bbox[1] + .5
+	    : rectOvalPtr->bbox[1] - .5);
     rectOvalPtr->header.y1 = tmp - bloat;
     dtmp = rectOvalPtr->bbox[2];
     if (dtmp < (rectOvalPtr->bbox[0] + 1)) {
 	dtmp = rectOvalPtr->bbox[0] + 1;
     }
-    tmp = (dtmp >= 0) ? dtmp + .5 : dtmp - .5;
+    tmp = (int) ((dtmp >= 0) ? dtmp + .5 : dtmp - .5);
     rectOvalPtr->header.x2 = tmp + bloat;
     dtmp = rectOvalPtr->bbox[3];
     if (dtmp < (rectOvalPtr->bbox[1] + 1)) {
 	dtmp = rectOvalPtr->bbox[1] + 1;
     }
-    tmp = (dtmp >= 0) ? dtmp + .5 : dtmp - .5;
+    tmp = (int) ((dtmp >= 0) ? dtmp + .5 : dtmp - .5);
     rectOvalPtr->header.y2 = tmp + bloat;
 }
 
@@ -560,7 +560,7 @@ DisplayRectOval(canvas, itemPtr, display, drawable, x, y, width, height)
 	    Tk_CanvasSetStippleOrigin(canvas, rectOvalPtr->fillGC);
 	}
 	if (rectOvalPtr->header.typePtr == &tkRectangleType) {
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
 	  if (rectOvalPtr->rounded>0)
 	    {
 	      int r=rectOvalPtr->rounded;
@@ -610,7 +610,7 @@ DisplayRectOval(canvas, itemPtr, display, drawable, x, y, width, height)
     }
     if (rectOvalPtr->outlineGC != None) {
 	if (rectOvalPtr->header.typePtr == &tkRectangleType) {
-#ifdef FULL_STEP
+#ifdef ENABLE_STEP
 	  if (rectOvalPtr->rounded>0)
 	    {
 	      int r=rectOvalPtr->rounded;
